Package: qol
Title: Powerful 'SAS' Inspired Concepts for more Efficient Bigger
        Outputs
Version: 1.3.4
Authors@R: 
    person("Tim", "Siebenmorgen", email = "qol_package@proton.me", role = c("aut", "cre", "cph"))
Description: The main goal is to make descriptive evaluations easier to create bigger and more complex outputs in less time with less code. Introducing format containers with multilabels <https://documentation.sas.com/doc/en/pgmsascdc/v_067/proc/p06ciqes4eaqo6n0zyqtz9p21nfb.htm>, a more powerful summarise which is capable to output every possible combination of the provided grouping variables in one go <https://documentation.sas.com/doc/en/pgmsascdc/v_067/proc/p0jvbbqkt0gs2cn1lo4zndbqs1pe.htm>, tabulation functions which can create any table in different styles <https://documentation.sas.com/doc/en/pgmsascdc/v_067/proc/n1ql5xnu0k3kdtn11gwa5hc7u435.htm> and other more readable functions. The code is optimized to work fast even with datasets of over a million observations.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://github.com/s3rdia/qol, https://s3rdia.github.io/qol/,
        https://deepwiki.com/s3rdia/qol,
        https://s3rdia.github.io/qol_blog/
BugReports: https://github.com/s3rdia/qol/issues
Imports: data.table (>= 1.17.8), collapse (>= 2.1.2), openxlsx2 (>=
        1.29), fst (>= 0.9.8)
Depends: R (>= 4.1.0)
RoxygenNote: 7.3.3
Suggests: tinytest (>= 1.4.3), yyjsonr (>= 0.1.22), rstudioapi (>=
        0.19.0)
Config/tinytest/edition: 1
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-08-20 18:32:27 UTC; User
Author: Tim Siebenmorgen [aut, cre, cph]
Maintainer: Tim Siebenmorgen <qol_package@proton.me>
Repository: CRAN
Date/Publication: 2026-08-21 05:41:02 UTC
Built: R 4.6.0; ; 2026-08-21 07:31:18 UTC; unix
