# Generated by roxygen2: do not edit by hand

export(add_extension)
export(add_variable_range)
export(any_table)
export(apply_macro)
export(args_to_char)
export(build_master)
export(build_rstheme)
export(check_required_package)
export(check_weight)
export(close_file)
export(code_statistics)
export(combine_into_workbook)
export(compute.)
export(concat)
export(content_report)
export(convert_factor)
export(convert_numeric)
export(convert_square_brackets)
export(create_table_of_contents)
export(crosstabs)
export(discrete_format)
export(do_if)
export(dots_to_char)
export(drop_type_vars)
export(dropp)
export(dummy_data)
export(else.)
export(else_do)
export(else_if.)
export(end_all_do)
export(end_do)
export(excel_output_style)
export(expand_formats)
export(export_data)
export(export_multi)
export(export_with_style)
export(first_row_as_names)
export(free_memory)
export(frequencies)
export(fuse_variables)
export(get_duplicate_var_count)
export(get_duplicate_var_names)
export(get_excel_range)
export(get_footnotes)
export(get_labels)
export(get_message_stack)
export(get_monitor)
export(get_na.rm)
export(get_origin_as_char)
export(get_output)
export(get_print)
export(get_print_miss)
export(get_style_options)
export(get_threads)
export(get_titles)
export(hex_to_256)
export(hex_to_ansi)
export(if.)
export(ifelse_multi)
export(import_data)
export(import_multi)
export(interval_format)
export(inverse)
export(keep)
export(libname)
export(load_file)
export(load_file_multi)
export(macro)
export(mark_case)
export(modify_number_formats)
export(modify_output_style)
export(monitor_end)
export(monitor_next)
export(monitor_plot)
export(monitor_start)
export(multi_join)
export(number_format_style)
export(part_of_df)
export(print_closing)
export(print_headline)
export(print_message)
export(print_stack_as_messages)
export(print_start_message)
export(print_step)
export(qol_chat)
export(qol_news)
export(recode.)
export(recode_multi)
export(remove_blanks)
export(remove_doubled_values)
export(remove_stat_extension)
export(rename_multi)
export(rename_pattern)
export(replace_except)
export(report_test_results)
export(reset_qol_options)
export(reset_style_options)
export(resolve_intersection)
export(retain_stat)
export(retain_value)
export(retain_variables)
export(round_multi)
export(round_values)
export(row_calculation)
export(run_folder)
export(run_project)
export(run_project_parallel)
export(run_script)
export(running_number)
export(save_file)
export(save_file_multi)
export(set_footnotes)
export(set_labels)
export(set_monitor)
export(set_na.rm)
export(set_no_color)
export(set_no_print)
export(set_output)
export(set_print)
export(set_print_miss)
export(set_style_options)
export(set_threads)
export(set_titles)
export(set_up_custom_message)
export(setcolorder_by_pattern)
export(sort_plus)
export(split_by)
export(stack_data)
export(sub_string)
export(summarise_plus)
export(test_package)
export(test_single_file)
export(transpose_plus)
export(vars_between)
export(where.)
