% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{qol_news}
\alias{qol_news}
\title{Go To GitHub NEWS Page}
\usage{
qol_news()
}
\value{
URL.
}
\description{
Opens browser and goes to the Github NEWS page
}
