% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{libname}
\alias{libname}
\title{Check If Path Exists And Retrieve Files}
\usage{
libname(path, get_files = FALSE, extensions = NULL, recursive = FALSE)
}
\arguments{
\item{path}{A folder path.}

\item{get_files}{FALSE by default. If TRUE returns a named character vector containing
file paths.}

\item{extensions}{Specify file extensions to be kept in the list when retrieving files.}

\item{recursive}{FALSE by default. If TRUE scans the provided folder for additional files
inside sub folders. Only work in combination with get_files = TRUE.}
}
\value{
Returns the given file path or a named character vector containing file paths.
}
\description{
\code{\link[=libname]{libname()}} checks if a given path exists and writes a message in the console accordingly.
Optional all files from the given path can be retrieved as a named character vector.
}
\examples{
my_path   <- libname("C:/My_Path/")
file_list <- libname("C:/My_Path/", get_files = TRUE)

}
